/*********************************************************************
 * 
 *   Copyright (C) 2001, 2002 Torsten Marek
 *
 * Filename:      tageditor.hpp
 * Description:   pure virtual base class for all tag editor classes
 * Author:        Torsten Marek <shlomme@gmx.net>
 *                
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 ********************************************************************/

#ifndef TAGEDITOR_HPP
#define TAGEDITOR_HPP

#include <string>

using namespace std;

enum FieldID {
	Artist,
	Title,
	Album,
	Year,
	Track,
	Genre,
	Comment
};

class TagEditor
{
public:
    // maybe add flag for readonly access
    TagEditor(const char* n) : fname(n)
	{
		tag_altered = 0;
		tag_insane = 0;
        length = bitrate = 0;
	}
	virtual ~TagEditor() 
	{
	}
    // return extension of the file
    virtual string extension() const = 0;
    // set value of field
    virtual int setField(FieldID, const string&) = 0;
    // get value of field
	virtual const string getField(FieldID) const = 0;
    // remove field from tag
	virtual int removeField(FieldID) = 0;
    // returns true if fields exists
    virtual bool existingField(FieldID) const = 0;
    // write modified tag back to file
	virtual int writeTag() = 0;
    // overloaded operator[], works like setField() and getField()
    // NOTE: If FieldID is invalid, the comment field is used instead.
    //       This behaviour may change in the future!
#ifndef PYTHON
	virtual string& operator[](FieldID) = 0;
                          
#else
    // workaround for the python bindings because sip can't (AFAIK)
    // handle overloaded operators, so it fails if they are pure
    // virtual. Normal plugins must not define PYTHON
    virtual string& operator[](FieldID) 
    {
    }
#endif
    virtual string& operator[](const string& s)
    {
        return operator[](mapToID(s));
    }
    // write all tag info to stdout (for debug reasons only, merely uninteresting)
    virtual void printTagInfo() const = 0;
    // if tag was read correctly
    bool isOK() const
	{
		return (tag_insane == 0);
	}
    const string& filename() const
	{
		return fname;
	}
	bool altered() const
	{
		return (tag_altered != 0);
	}
    void setAltered(bool state)
    {
        tag_altered = state;
    }
    // returns the length of the track in seconds
    unsigned int getLength() const
	{
		return length;
	}
    unsigned int getBitrate() const
    {
        return bitrate;
    }
    FieldID mapToID(const string& str) const
    {
        if(str == "Artist")
            return Artist;
        else if(str == "Title")
            return Title;
        else if(str == "Album")
            return Album;
        else if(str == "Year")
            return Year;
        else if(str == "Track")
            return Track;
        else if(str == "Genre")
            return Genre;
        else if(str == "Comment")
            return Comment;
        else
            return (FieldID)-1;
    }
protected:
    string fname;
	bool tag_altered;
	bool tag_insane;
	unsigned int length;
    unsigned int bitrate;
};

#endif // TAGEDITOR_HPP
